# Generated by Django 5.2.4 on 2025-07-31 07:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("aldryn_forms", "0022_urlfieldplugin"),
    ]

    operations = [
        migrations.AlterField(
            model_name="formplugin",
            name="action_backend",
            field=models.CharField(
                choices=[
                    ("none", "No action"),
                    ("email_only", "Only send email"),
                    ("default", "Save to site administration and send email"),
                ],
                default="default",
                max_length=15,
                verbose_name="Action backend",
            ),
        ),
        migrations.AlterField(
            model_name="submittedtobesent",
            name="language",
            field=models.CharField(
                choices=[("en", "English")],
                default="en",
                max_length=10,
                verbose_name="form language",
            ),
        ),
    ]



# Generated by Django 4.2.20 on 2025-04-29 13:38

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0022_formplugin_redirect_to'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='formplugin',
            name='redirect_page',
        ),
        migrations.RemoveField(
            model_name='formplugin',
            name='redirect_type',
        ),
        migrations.RemoveField(
            model_name='formplugin',
            name='url',
        ),
    ]



# Generated by Django 4.2.20 on 2025-03-21 08:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0020_webhook'),
    ]

    operations = [
        migrations.AddField(
            model_name='formsubmission',
            name='honeypot_filled',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='submittedtobesent',
            name='honeypot_filled',
            field=models.BooleanField(default=False),
        ),
    ]



import re

from django import template
from django.contrib.sites.models import Site
from django.utils.html import escape
from django.utils.safestring import mark_safe


register = template.Library()

link_pattern = None


@register.filter
def media_filer_public_link(value):
    global link_pattern

    if not isinstance(value, str):
        return value

    if link_pattern is None:
        hostnames = "|".join(Site.objects.values_list('domain', flat=True))
        link_pattern = f"^https?://({hostnames})/media/filer_public/"

    content = []
    for word in re.split(r"(\s+)", value):
        if re.match(link_pattern, word):
            filename = escape(word.split("/")[-1])
            word = f"""<a href="{word}" target="_blank">{filename}</a>"""
        else:
            word = escape(word)
        content.append(word)

    return mark_safe("".join(content))



# Generated by Django 5.2.5 on 2025-09-02 09:08

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("aldryn_forms", "0023_alter_formplugin_action_backend_and_more"),
        ("aldryn_forms", "0023_remove_formplugin_redirect_page_and_more"),
    ]

    operations = []



"""
The AldrynFormsPageField class, unlike the PageField class, also displays unpublished pages.
"""
from djangocms_link.fields import LinkField, LinkFormField

from .widgets import LinkWidget


class AldrynLinkFormField(LinkFormField):

    widget = LinkWidget


class AldrynFormsLinkField(LinkField):

    def formfield(self, **kwargs):
        kwargs.setdefault("form_class", AldrynLinkFormField)
        return super().formfield(**kwargs)



# Generated by Django 5.2.6 on 2025-10-21 06:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("aldryn_forms", "0024_merge_20250902_1108"),
    ]

    operations = [
        migrations.AlterField(
            model_name="formplugin",
            name="action_backend",
            field=models.CharField(
                default="default", max_length=15, verbose_name="Action backend"
            ),
        ),
        migrations.AlterField(
            model_name="formsubmission",
            name="language",
            field=models.CharField(max_length=10, verbose_name="form language"),
        ),
        migrations.AlterField(
            model_name="submittedtobesent",
            name="language",
            field=models.CharField(max_length=10, verbose_name="form language"),
        ),
    ]



DEFAULT_ALDRYN_FORMS_ACTION_BACKENDS = {
    'default': 'aldryn_forms.action_backends.DefaultAction',
    'email_only': 'aldryn_forms.action_backends.EmailAction',
    'none': 'aldryn_forms.action_backends.NoAction',
}
ALDRYN_FORMS_ACTION_BACKEND_KEY_MAX_SIZE = 15

ALDRYN_FORMS_POST_IDENT_NAME = "aldryn_form_post_ident"
MAX_IDENT_SIZE = 64

ALDRYN_FORMS_MULTIPLE_SUBMISSION_DURATION = "ALDRYN_FORMS_MULTIPLE_SUBMISSION_DURATION"

EMAIL_REPLY_TO = "email_reply_to"

WEBHOOK_METHODS = (
    ('post', 'POST'),
    ('json', 'JSON'),
)

TRANSFORM_SCHEMA = {
    "type": "array",
    "items": {
        "$ref": "#/$defs/field"
    },
    "$defs": {
        "field": {
            "type": "object",
            "required": ["dest"],
            "oneOf": [
                {"required": ["src"]},
                {"required": ["value"]},
                {"required": ["fnc"]},
            ],
            "properties": {
                "dest": {"type": "string"},
                "value": {"type": "string"},
                "src": {"type": ["string", "array"]},
                "match": {"type": ["string", "array"]},
                "fetcher": {"enum": ["first", "all", "text"]},
                "sep": {"type": "string"},
                "fnc": {"type": "string"},
                "params": {"type": "object"},
            }
        }
    }
}



INSTALLED_APPS = [
    "django.contrib.sites",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.auth",
    "django.contrib.admin",
    "cms",
    "djangocms_alias",
    "menus",
    "easy_thumbnails",
    "treebeard",
    "filer",
    "aldryn_forms",
]

MIDDLEWARE = [
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
]

TEMPLATES = [{
    'BACKEND': 'django.template.backends.django.DjangoTemplates',
    'OPTIONS': {
        'context_processors': [
            'django.template.context_processors.request',
            'django.contrib.auth.context_processors.auth',
            'django.contrib.messages.context_processors.messages',
            'django.template.context_processors.i18n',
            'cms.context_processors.cms_settings',
        ],
        'loaders': [
            'django.template.loaders.filesystem.Loader',
            'django.template.loaders.app_directories.Loader'
        ]
    }
}]

LANGUAGES = [
    ("en", "English"),
]
LANGUAGE_CODE = "en"

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"



from typing import Dict


dataType = Dict[str, str]


def remove_identical_value(rule: dataType, src: dataType, dest: dataType) -> None:
    """Remove value if it is identical to another."""
    data = [dest.get(key) for key in rule.get("params", {}).get("fields", [])]
    if len(set(data)) < len(data) and rule["dest"] in dest:
        del dest[rule["dest"]]



from django.apps import AppConfig


class EmailNotifications(AppConfig):
    name = 'aldryn_forms.contrib.email_notifications'
    verbose_name = 'Email notifications'
    default_auto_field = "django.db.models.BigAutoField"



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0006_auto_20160821_1039'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailfieldplugin',
            name='custom_classes',
            field=models.CharField(max_length=255, verbose_name='custom css classes', blank=True),
        ),
        migrations.AlterField(
            model_name='emailfieldplugin',
            name='email_subject',
            field=models.CharField(default='', help_text='Used as the email subject when email_send_notification is checked.', max_length=255, verbose_name='email subject', blank=True),
        ),
        migrations.AlterField(
            model_name='emailfieldplugin',
            name='label',
            field=models.CharField(max_length=255, verbose_name='Label', blank=True),
        ),
        migrations.AlterField(
            model_name='emailfieldplugin',
            name='placeholder_text',
            field=models.CharField(help_text='Default text in a form. Disappears when user starts typing. Example: "email@exmaple.com"', max_length=255, verbose_name='Placeholder text', blank=True),
        ),
        migrations.AlterField(
            model_name='fieldplugin',
            name='custom_classes',
            field=models.CharField(max_length=255, verbose_name='custom css classes', blank=True),
        ),
        migrations.AlterField(
            model_name='fieldplugin',
            name='label',
            field=models.CharField(max_length=255, verbose_name='Label', blank=True),
        ),
        migrations.AlterField(
            model_name='fieldplugin',
            name='placeholder_text',
            field=models.CharField(help_text='Default text in a form. Disappears when user starts typing. Example: "email@exmaple.com"', max_length=255, verbose_name='Placeholder text', blank=True),
        ),
        migrations.AlterField(
            model_name='fieldsetplugin',
            name='custom_classes',
            field=models.CharField(max_length=255, verbose_name='custom css classes', blank=True),
        ),
        migrations.AlterField(
            model_name='fieldsetplugin',
            name='legend',
            field=models.CharField(max_length=255, verbose_name='Legend', blank=True),
        ),
        migrations.AlterField(
            model_name='fileuploadfieldplugin',
            name='custom_classes',
            field=models.CharField(max_length=255, verbose_name='custom css classes', blank=True),
        ),
        migrations.AlterField(
            model_name='fileuploadfieldplugin',
            name='label',
            field=models.CharField(max_length=255, verbose_name='Label', blank=True),
        ),
        migrations.AlterField(
            model_name='fileuploadfieldplugin',
            name='placeholder_text',
            field=models.CharField(help_text='Default text in a form. Disappears when user starts typing. Example: "email@exmaple.com"', max_length=255, verbose_name='Placeholder text', blank=True),
        ),
        migrations.AlterField(
            model_name='formbuttonplugin',
            name='custom_classes',
            field=models.CharField(max_length=255, verbose_name='custom css classes', blank=True),
        ),
        migrations.AlterField(
            model_name='formbuttonplugin',
            name='label',
            field=models.CharField(max_length=255, verbose_name='Label'),
        ),
        migrations.AlterField(
            model_name='formplugin',
            name='custom_classes',
            field=models.CharField(max_length=255, verbose_name='custom css classes', blank=True),
        ),
        migrations.AlterField(
            model_name='formplugin',
            name='form_template',
            field=models.CharField(default='aldryn_forms/form.html', max_length=255, verbose_name='form template', choices=[('aldryn_forms/form.html', 'Default'), ('aldryn_forms/horizontal/form.html', 'Horizontal')]),
        ),
        migrations.AlterField(
            model_name='formplugin',
            name='name',
            field=models.CharField(help_text='Used to filter out form submissions.', max_length=255, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='formsubmission',
            name='name',
            field=models.CharField(verbose_name='form name', max_length=255, editable=False, db_index=True),
        ),
        migrations.AlterField(
            model_name='imageuploadfieldplugin',
            name='custom_classes',
            field=models.CharField(max_length=255, verbose_name='custom css classes', blank=True),
        ),
        migrations.AlterField(
            model_name='imageuploadfieldplugin',
            name='label',
            field=models.CharField(max_length=255, verbose_name='Label', blank=True),
        ),
        migrations.AlterField(
            model_name='imageuploadfieldplugin',
            name='placeholder_text',
            field=models.CharField(help_text='Default text in a form. Disappears when user starts typing. Example: "email@exmaple.com"', max_length=255, verbose_name='Placeholder text', blank=True),
        ),
        migrations.AlterField(
            model_name='option',
            name='value',
            field=models.CharField(max_length=255, verbose_name='Value'),
        ),
        migrations.AlterField(
            model_name='textareafieldplugin',
            name='custom_classes',
            field=models.CharField(max_length=255, verbose_name='custom css classes', blank=True),
        ),
        migrations.AlterField(
            model_name='textareafieldplugin',
            name='label',
            field=models.CharField(max_length=255, verbose_name='Label', blank=True),
        ),
        migrations.AlterField(
            model_name='textareafieldplugin',
            name='placeholder_text',
            field=models.CharField(help_text='Default text in a form. Disappears when user starts typing. Example: "email@exmaple.com"', max_length=255, verbose_name='Placeholder text', blank=True),
        ),
    ]



import logging
import smtplib
from typing import TYPE_CHECKING, Dict, ItemsView, List

from django import forms
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.forms.forms import NON_FIELD_ERRORS
from django.template import Context, Template
from django.utils.module_loading import import_string
from django.utils.translation import get_language

from cms.models import CMSPlugin
from cms.plugin_pool import plugin_pool
from cms.utils.plugins import downcast_plugins

from emailit.api import send_mail
from emailit.utils import get_template_names


try:
    from constance import config as constance_config
except ModuleNotFoundError:
    constance_config = None


from .action_backends_base import BaseAction
from .compat import build_plugin_tree
from .constants import (
    ALDRYN_FORMS_ACTION_BACKEND_KEY_MAX_SIZE, ALDRYN_FORMS_POST_IDENT_NAME, DEFAULT_ALDRYN_FORMS_ACTION_BACKENDS,
    EMAIL_REPLY_TO,
)
from .validators import is_valid_recipient


if TYPE_CHECKING:  # pragma: no cover
    from .models import FormSubmissionBase, Recipient, SerializedFormField

logger = logging.getLogger(__name__)


def get_action_backends():
    base_error_msg = 'Invalid settings.ALDRYN_FORMS_ACTION_BACKENDS.'
    max_key_size = ALDRYN_FORMS_ACTION_BACKEND_KEY_MAX_SIZE

    try:
        backends = settings.ALDRYN_FORMS_ACTION_BACKENDS
    except AttributeError:
        backends = DEFAULT_ALDRYN_FORMS_ACTION_BACKENDS

    try:
        backends = {k: import_string(v) for k, v in backends.items()}
    except ImportError as e:
        raise ImproperlyConfigured(f'{base_error_msg} {e}')

    if any(len(key) > max_key_size for key in backends):
        raise ImproperlyConfigured(
            f'{base_error_msg} Ensure all keys are no longer than {max_key_size} characters.'
        )

    if not all(issubclass(klass, BaseAction) for klass in backends.values()):
        raise ImproperlyConfigured(
            '{} All classes must derive from aldryn_forms.action_backends_base.BaseAction'
            .format(base_error_msg)
        )

    if 'default' not in backends.keys():
        raise ImproperlyConfigured(f'{base_error_msg} Key "default" is missing.')

    try:
        [x() for x in backends.values()]  # check abstract base classes sanity
    except TypeError as e:
        raise ImproperlyConfigured(f'{base_error_msg} {e}')
    return backends


def action_backend_choices(*args, **kwargs):
    choices = tuple((key, klass.verbose_name) for key, klass in get_action_backends().items())
    return sorted(choices, key=lambda x: x[1])


def get_user_model():
    """
    Wrapper for get_user_model with compatibility for 1.5
    """
    # Notice these imports happen here to be compatible with django 1.7
    try:
        from django.contrib.auth import get_user_model as _get_user_model
    except ImportError:  # django < 1.5
        from django.contrib.auth.models import User
    else:
        User = _get_user_model()
    return User


def get_nested_plugins(parent_plugin, include_self=False):
    """
    Returns a flat list of plugins from parent_plugin. Replace AliasPlugin by descendants.
    """
    AliasPlugin = plugin_pool.get_plugin("Alias")

    found_plugins = []

    if include_self:
        found_plugins.append(parent_plugin)

    child_plugins = parent_plugin.get_children()

    for plugin in child_plugins:
        if issubclass(plugin.get_plugin_class(), AliasPlugin):
            if hasattr(plugin, "plugin"):
                found_plugins.extend(plugin.plugin.get_descendants())
            else:
                found_plugins.extend(plugin.get_descendants())
        else:
            found_plugins.extend(get_nested_plugins(plugin, include_self=True))

    return found_plugins


def get_plugin_tree(model, **kwargs):
    """
    Plugins in django CMS are highly related to a placeholder.

    This function builds a plugin tree for a plugin with no placeholder context.

    Makes as many database queries as many levels are in the tree.

    This is ok as forms shouldn't form very deep trees.
    """
    plugin = model.objects.get(**kwargs)
    plugin.parent = None
    current_level = [plugin]
    plugin_list = [plugin]
    while get_next_level(current_level).exists():
        current_level = get_next_level(current_level)
        current_level = downcast_plugins(current_level)
        plugin_list += current_level
    return build_plugin_tree(plugin_list)[0]


def get_next_level(current_level):
    all_plugins = CMSPlugin.objects.all()
    return all_plugins.filter(parent__in=[x.pk for x in current_level])


def add_form_error(form, message, field=NON_FIELD_ERRORS):
    try:
        form._errors[field].append(message)
    except KeyError:
        form._errors[field] = form.error_class([message])


def send_postponed_notifications(instance: "FormSubmissionBase") -> bool:
    """Send postponed notifications."""
    recipients = [user for user in instance.get_recipients() if is_valid_recipient(user.email)]
    if not recipients:
        return True
    form_data = instance.get_form_data()
    cleaned_data = [(field.name, field.value) for field in form_data]
    return send_email(recipients, instance, form_data, cleaned_data)


def send_email(
    recipients: List["Recipient"],
    instance: "FormSubmissionBase",
    form_data: List["SerializedFormField"],
    cleaned_data: ItemsView
) -> bool:
    """Send email."""
    context = {
        'form_name': instance.name,
        'form_data': form_data,
        'form_plugin': instance,
        'form_values': {sf.name: sf.value for sf in form_data},
    }
    subject_template_base = getattr(settings, 'ALDRYN_FORMS_EMAIL_SUBJECT_TEMPLATES_BASE',
                                    getattr(settings, 'ALDRYN_FORMS_EMAIL_TEMPLATES_BASE', None))
    if subject_template_base:
        language = instance.language or get_language()
        subject_templates = get_template_names(language, subject_template_base, 'subject', 'txt')
    else:
        subject_templates = None

    subject = None
    if constance_config is not None:
        template_string = getattr(constance_config, f"ALDRYN_FORMS_EMAIL_SUBJECT_{instance.language.upper()}", None)
        if template_string is not None:
            subject = Template(template_string).render(Context(context))

    reply_to = []
    for name, value in cleaned_data:
        if name == EMAIL_REPLY_TO:
            reply_to.append(value)
    try:
        send_mail(
            recipients=[user.email for user in recipients],
            context=context,
            template_base=getattr(
                settings, 'ALDRYN_FORMS_EMAIL_TEMPLATES_BASE', 'aldryn_forms/emails/notification'),
            subject=subject,
            subject_templates=subject_templates,
            language=instance.language,
            reply_to=reply_to,
        )
    except smtplib.SMTPException as err:
        logger.error(err)
        return False
    return True


def get_serialized_fields(form: forms.Form) -> Dict[str, str]:
    """Get serialized fields. Skip honeypost and ident field."""
    fields_as_dicts: List[Dict[str, str]] = []
    for field in form.get_serialized_fields(is_confirmation=False):
        item = field._asdict()
        if item["plugin_type"] == "HoneypotField" and not item["value"]:
            continue
        if field.name == ALDRYN_FORMS_POST_IDENT_NAME:
            continue
        fields_as_dicts.append(item)
    return fields_as_dicts



import json
import logging
import re
from typing import TYPE_CHECKING, Any, Dict, List, Optional, Union

from django.db.models import ManyToManyField
from django.utils.module_loading import import_string

import jq
import requests
from requests.exceptions import RequestException


dataType = Dict[str, Any]

if TYPE_CHECKING:  # pragma: no cover
    from aldryn_forms.models import FormSubmission, FormSubmissionBase, Webhook

logger = logging.getLogger(__name__)


def send_to_webhook(url: str, method: str, data: dataType) -> requests.Response:
    """Send data to URL as POST."""
    if method == "JSON":
        response = requests.post(url, json.dumps(data), headers={"Content-Type": "application/json"})
    else:
        response = requests.post(url, data)
    response.raise_for_status()
    return response


def trigger_webhooks(webhooks: ManyToManyField, instance: "FormSubmissionBase", hostname: str) -> None:
    """Trigger webhooks and send them the instance data."""
    from aldryn_forms.api.serializers import FormSubmissionSerializer
    serializer = FormSubmissionSerializer(instance, context={"hostname": hostname})

    for hook in webhooks.all():
        data = transform_data(hook.transform, serializer.data)
        logger.debug(data)
        try:
            send_to_webhook(hook.url, hook.method, data)
        except RequestException as err:
            logger.error(f"{hook.url} {err}")


def transform_data(transform: Optional[List[dataType]], data: dataType) -> dataType:
    """Transform data according to rules."""
    if transform is None:
        return data
    out: dataType = {}
    for rule in transform:
        if "value" in rule:
            out[rule["dest"]] = rule["value"]
        elif "fnc" in rule:
            try:
                import_string(rule["fnc"])(rule, data, out)
            except Exception as err:
                logger.error(f"{rule['fnc']} {err}")
        else:
            chunks = []
            src = [rule["src"]] if isinstance(rule["src"], str) else rule["src"]
            for query in src:
                try:
                    input = jq.compile(query).input(data)
                except ValueError as err:
                    logger.error(f"{query} {err}")
                    continue
                try:
                    value = getattr(input, rule.get("fetcher", "first"))()
                except (StopIteration, ValueError) as err:
                    logger.debug(f"StopIteration {query} {err}")
                    continue
                chunks.append(str(value))
            if chunks:
                value = rule.get("sep", " ").join(chunks)
                if "match" in rule:
                    value = process_match(rule["match"], value)
                if value:
                    out[rule["dest"]] = value
    return out


def process_match(pattern: Union[str, List], value: str) -> str:
    """Process match."""
    flags = 0
    separator = " "
    if isinstance(pattern, list):
        if len(pattern) > 1:
            try:
                for flg in pattern[1]:
                    flags |= getattr(re, flg)
            except AttributeError as err:
                logger.error(f"{flg} {err}")
        if len(pattern) > 2:
            separator = pattern[2]
        pattern = pattern[0]
    try:
        match = re.match(pattern, value, flags)
    except (AttributeError, re.error) as err:
        logger.error(f"{pattern} {err}")
        return value
    if match is None:
        return value
    return separator.join(match.groups())


def collect_submissions_data(webhook: "Webhook", submissions: "FormSubmission", hostname: str) -> List[Dict[str, str]]:
    """Collect submissions data."""
    from aldryn_forms.api.serializers import FormSubmissionSerializer

    response = []
    for instance in submissions.all():
        serializer = FormSubmissionSerializer(instance, context={"hostname": hostname})
        data = transform_data(webhook.transform, serializer.data)
        response.append(data)

    return response


def send_submissions_data(webhook: "Webhook", submissions: "FormSubmission", hostname: str) -> None:
    """Send submissions data to webhook."""
    from aldryn_forms.api.serializers import FormSubmissionSerializer

    for instance in submissions.all():
        serializer = FormSubmissionSerializer(instance, context={"hostname": hostname})
        data = transform_data(webhook.transform, serializer.data)
        logger.debug(data)
        try:
            send_to_webhook(webhook.url, webhook.method, data)
        except RequestException as err:
            logger.error(f"{webhook.url} {err}")



# Generated by Django 4.2.20 on 2025-07-30 09:39

import django.db.models.deletion
from django.db import migrations, models

import djangocms_attributes_field.fields


class Migration(migrations.Migration):

    dependencies = [
        ("aldryn_forms", "0021_formsubmission_honeypot_filled"),
    ]

    operations = [
        migrations.CreateModel(
            name="URLFieldPlugin",
            fields=[
                (
                    "name",
                    models.CharField(
                        blank=True,
                        help_text="Used to set the field name",
                        max_length=255,
                        verbose_name="Name",
                    ),
                ),
                (
                    "label",
                    models.CharField(blank=True, max_length=255, verbose_name="Label"),
                ),
                (
                    "required",
                    models.BooleanField(
                        default=False, verbose_name="Field is required"
                    ),
                ),
                (
                    "required_message",
                    models.TextField(
                        blank=True,
                        help_text='Error message displayed if the required field is left empty. Default: "This field is required".',
                        null=True,
                        verbose_name="Error message",
                    ),
                ),
                (
                    "placeholder_text",
                    models.CharField(
                        blank=True,
                        help_text='Default text in a form. Disappears when user starts typing. Example: "email@example.com"',
                        max_length=255,
                        verbose_name="Placeholder text",
                    ),
                ),
                (
                    "help_text",
                    models.TextField(
                        blank=True,
                        help_text="Explanatory text displayed next to input field. Just like this one.",
                        null=True,
                        verbose_name="Help text",
                    ),
                ),
                (
                    "attributes",
                    djangocms_attributes_field.fields.AttributesField(
                        blank=True, default=dict, verbose_name="Attributes"
                    ),
                ),
                (
                    "min_value",
                    models.PositiveIntegerField(
                        blank=True, null=True, verbose_name="Min value"
                    ),
                ),
                (
                    "max_value",
                    models.PositiveIntegerField(
                        blank=True, null=True, verbose_name="Max value"
                    ),
                ),
                (
                    "initial_value",
                    models.CharField(
                        blank=True,
                        help_text="Default value of field.",
                        max_length=255,
                        verbose_name="Initial value",
                    ),
                ),
                (
                    "custom_classes",
                    models.CharField(
                        blank=True, max_length=255, verbose_name="custom css classes"
                    ),
                ),
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="%(app_label)s_%(class)s",
                        serialize=False,
                        to="cms.cmsplugin",
                    ),
                ),
                (
                    "list",
                    models.TextField(
                        blank=None,
                        help_text="A list of predefined url values. Each url is on a separate line. The first word is the url. The next (optional) words are a description for the url.",
                        null=True,
                        verbose_name="List of urls",
                    ),
                ),
                (
                    "pattern",
                    models.CharField(
                        blank=None,
                        help_text="A regular expression to match for the value. It must be a valid JavaScript regular expression.",
                        null=True,
                        max_length=255,
                        verbose_name="Pattern",
                    ),
                ),
                (
                    "readonly",
                    models.BooleanField(
                        blank=None,
                        help_text="The field cannot be edited by the user.",
                        null=True,
                        verbose_name="Read only",
                    ),
                ),
                (
                    "size",
                    models.SmallIntegerField(
                        blank=None,
                        help_text="How many characters wide the input field should be. The default value is 20.",
                        null=True,
                        verbose_name="Size",
                    ),
                ),
                (
                    "spellcheck",
                    models.BooleanField(
                        blank=None,
                        help_text="To enable spell-checking for an element.",
                        null=True,
                        verbose_name="Spellcheck",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("cms.cmsplugin",),
        ),
    ]



import re
from email.utils import formataddr
from typing import Callable
from urllib.parse import urlencode

from django import forms
from django.conf import settings
from django.contrib import admin, messages
from django.contrib.sites.models import Site
from django.db.models.query import QuerySet
from django.http import HttpRequest, HttpResponse, HttpResponseRedirect, JsonResponse
from django.template.loader import render_to_string
from django.template.response import TemplateResponse
from django.urls import reverse
from django.utils.encoding import force_str
from django.utils.module_loading import import_string
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _

from import_export.fields import Field
from import_export.resources import Resource

from ..api.webhook import collect_submissions_data, send_submissions_data
from ..models import FormSubmission, Webhook
from .utils import PrettyJsonEncoder


str_dunder_method = '__str__'


class FieldKey:
    """Field key is pair of names - parent, child."""

    def __init__(self, parent, child):
        self.parent = parent
        self.child = child

    def __str__(self):
        return "{}+{}".format(self.parent, self.child)


class AldrynFormExportField(Field):
    """AldrynForm export field."""

    def get_value(self, obj):
        if isinstance(self.attribute, FieldKey):
            return obj.get(self.attribute.parent, {}).get(self.attribute.child)
        return obj.get(self.attribute)


class BaseFormSubmissionAdmin(admin.ModelAdmin):
    date_hierarchy = 'sent_at'
    list_display = [
        str_dunder_method, 'sent_at', 'display_honeypot_filled', 'display_post_ident', 'language', 'display_data'
    ]
    list_filter = [
        'name',
        'language',
        'sent_at',
        ("honeypot_filled", admin.BooleanFieldListFilter),
        ("post_ident", admin.BooleanFieldListFilter),
    ]
    search_fields = ["data"]
    actions = ["export_webhook", "send_webhook", "honeypot_filled_on", "honeypot_filled_off"]
    readonly_fields = [
        'name',
        'get_data_for_display',
        'language',
        'sent_at',
        'get_recipients_for_display',
        'post_ident',
        'webhooks',
    ]
    fieldsets = [
        (
            None,
            {
                "fields": [
                    "name", "get_data_for_display", "language", "sent_at", "webhooks", "form_url", "honeypot_filled"
                ]
            }
        )
    ]

    # (Field name, Field label, json data)
    export_fields = (
        ('name', _('form name'), False),
        ('language', _('form language'), False),
        ('sent_at', _('sent at'), False),
        ('data', _('form data'), True),
        ('recipients', _('users notified'), True),
    )

    class Media:
        css = {
            "all": ["aldryn_forms/css/admin-form.css"],
        }

    def has_add_permission(self, request):
        return False

    def get_search_results(self, request, queryset, search_term):
        queryset, may_have_duplicates = super().get_search_results(request, queryset, search_term)
        if search_term:
            try:
                re.match(search_term, "")
                queryset |= self.model.objects.filter(data__regex=search_term)
            except Exception as err:
                messages.error(request, err)
        return queryset, may_have_duplicates

    def get_data_for_display(self, obj):
        data = obj.get_form_data()
        html = render_to_string(
            'admin/aldryn_forms/display/submission_data.html',
            {'data': data}
        )
        return html
    get_data_for_display.allow_tags = True
    get_data_for_display.short_description = _('data')

    def get_recipients(self, obj):
        recipients = obj.get_recipients()
        formatted = [formataddr((recipient.name, recipient.email))
                     for recipient in recipients]
        return formatted

    def get_recipients_for_display(self, obj):
        people_list = self.get_recipients(obj)
        html = render_to_string(
            'admin/aldryn_forms/display/recipients.html',
            {'people': people_list},
        )
        return html
    get_recipients_for_display.allow_tags = True
    get_recipients_for_display.short_description = _('people notified')

    def get_urls(self):
        from django.urls import path

        url_patterns = [
            path('export/', self.admin_site.admin_view(self.get_form_export_view()), name=self.get_admin_url('export')),
            path("webhook-export/", self.admin_site.admin_view(self.webhook_export), name="webhook_export"),
            path("webhook-send/", self.admin_site.admin_view(self.webhook_send), name="webhook_send"),
        ]
        return url_patterns + super(BaseFormSubmissionAdmin, self).get_urls()

    def get_admin_url(self, name):
        try:
            model_name = self.model._meta.model_name
        except AttributeError:
            # django <= 1.5 compat
            model_name = self.model._meta.module_name

        url_name = "%s_%s_%s" % (self.model._meta.app_label, model_name, name)
        return url_name

    def get_admin_context(self, form=None, title=None):
        opts = self.model._meta

        context = {
            'media': self.media,
            'has_change_permission': True,
            'opts': opts,
            'root_path': reverse('admin:index'),
            'current_app': self.admin_site.name,
            'app_label': opts.app_label,
        }

        if form:
            context['adminform'] = form
            context['media'] += form.media

        if title:
            context['original'] = title
        return context

    def get_form_export_view(self):
        raise NotImplementedError

    def export_field_parse_data(self, submission):
        """Parse export form field data."""
        fields, values = {}, {}
        for serialized_form_field in submission.get_form_data():
            if serialized_form_field.value:
                fields[serialized_form_field.field_id] = serialized_form_field.label
                values[serialized_form_field.field_id] = serialized_form_field.value
        return fields, values

    def export_field_parse_recipients(self, submission):
        """Parse export form field recipients."""
        fields, values = {'email': _("E-mail")}, {}
        for recipient in submission.get_recipients():
            values['email'] = recipient.email
            if recipient.name:
                values['name'] = recipient.name
                if 'name' not in fields:
                    fields['name'] = _('Name')
        return fields, values

    def export_dataset_and_labels(self, queryset):
        """Collect fields from JSON data."""
        dataset = []
        extra_field_labels = {}
        cols = {name: label for name, label, json_data in self.export_fields}
        unique_codes = []
        for submission in queryset:
            data_item = {}
            for field_name, field_label, field_json_data in self.export_fields:
                field_value = getattr(submission, field_name)
                if field_value is None or field_value == "":
                    continue
                if field_json_data:
                    fnc = getattr(self, "export_field_parse_{}".format(field_name), None)
                    if fnc is not None:
                        field_columns, field_values = fnc(submission)
                        data_item[field_name] = field_values
                        for name, label in field_columns.items():
                            field_key = FieldKey(field_name, name)
                            code = str(field_key)
                            if code not in unique_codes:
                                unique_codes.append(code)
                                if field_name not in extra_field_labels:
                                    extra_field_labels[field_name] = {}
                                extra_field_labels[field_name][field_key] = "{} / {}".format(cols[field_name], label)
                    else:
                        data_item[field_name] = field_value
                else:
                    data_item[field_name] = field_value
            dataset.append(data_item)
        return dataset, extra_field_labels

    def export_data(self, export_type, queryset):
        """Export data into format defined by export_type."""
        extra_field_labels = {}
        dataset, extra_field_labels = self.export_dataset_and_labels(queryset)

        fields = []
        headers = []
        for name, label, json_data in self.export_fields:
            if json_data:
                for code, label in extra_field_labels.get(name, {}).items():
                    fields.append(AldrynFormExportField(attribute=code))
                    headers.append(force_str(label))
            else:
                fields.append(AldrynFormExportField(attribute=name))
                headers.append(force_str(label))

        resource = Resource()
        resource.get_export_headers = lambda: headers
        for field in fields:
            resource.fields[force_str(field.attribute)] = field

        return getattr(resource.export(dataset), export_type)

    @admin.display(description=_("data"))
    def display_data(self, obj) -> str:
        if hasattr(settings, 'ALDRYN_FORMS_SUBMISSION_LIST_DISPLAY_FIELD'):
            submission_field = import_string(settings.ALDRYN_FORMS_SUBMISSION_LIST_DISPLAY_FIELD)
            return submission_field(obj)
        return ''

    @admin.display(boolean=True, description=_("Is spam"))
    def display_honeypot_filled(self, obj) -> bool:
        return obj.honeypot_filled

    @admin.display(boolean=True, description=_("Ready"))
    def display_post_ident(self, obj) -> bool:
        return obj.post_ident is None

    def get_select_webhook_form(self) -> forms.Form:
        return type("SelectWebhookForm", (forms.Form,), {
            "webhook": forms.ChoiceField(choices=Webhook.objects.values_list("pk", "name").order_by("name")),
        })

    def export_submissions_by_webhook(
        self, request: HttpRequest, submissions: FormSubmission, webhook: Webhook
    ) -> JsonResponse:
        site = Site.objects.first()
        data = collect_submissions_data(webhook, submissions, site.domain)
        response = JsonResponse({"data": data}, encoder=PrettyJsonEncoder, json_dumps_params={"ensure_ascii": False})
        filename = f"form-submissions-webhook-{slugify(webhook.name)}.json"
        response["Content-Disposition"] = f"attachment; filename={filename}"
        return response

    def send_submissions_data(
        self, request: HttpRequest, submissions: FormSubmission, webhook: Webhook
    ) -> HttpResponseRedirect:
        site = Site.objects.first()
        send_submissions_data(webhook, submissions, site.domain)
        messages.success(request, _("Data sending completed."))
        return HttpResponseRedirect(reverse("admin:aldryn_forms_formsubmission_changelist"))

    def process_webhook(self, request: HttpRequest, process_fnc: Callable, process_title: str) -> HttpResponse:
        ids = request.GET.get("ids", "")
        if not ids:
            messages.warning(
                request, "Items must be selected in order to perform actions on them. No items have been changed.")
            return HttpResponseRedirect(reverse("admin:aldryn_forms_formsubmission_changelist"))
        submissions = FormSubmission.objects.filter(pk__in=ids.split("."))
        SelectWebhookForm = self.get_select_webhook_form()
        if request.method == "POST":
            if submissions.count():
                form = SelectWebhookForm(request.POST)
                if form.is_valid():
                    webhook = Webhook.objects.get(pk=form.cleaned_data["webhook"])
                    return process_fnc(request, submissions, webhook)
                else:
                    messages.error(request, form.errors.as_text())
            else:
                messages.error(request, _("Missing items for processing."))
            return HttpResponseRedirect(reverse("admin:aldryn_forms_formsubmission_changelist"))
        else:
            form = SelectWebhookForm()
        data = {
            "ids": ids,
            "form": form,
            "submissins_size": submissions.count(),
            "process_title": process_title,
        }
        context = dict(self.admin_site.each_context(request), **data)
        return TemplateResponse(request, "admin/aldryn_forms/formsubmission/webhook_form.html", context)

    def webhook_export(self, request: HttpRequest) -> HttpResponse:
        return self.process_webhook(request, self.export_submissions_by_webhook, _("Export data via webhook"))

    def webhook_send(self, request: HttpRequest) -> HttpResponse:
        return self.process_webhook(request, self.send_submissions_data, _("Send data via webhook"))

    def process_response_redirect(self, queryset: QuerySet, path_name: str) -> HttpResponseRedirect:
        selected = queryset.values_list("pk", flat=True)
        params = urlencode({"ids": ".".join([str(pk) for pk in selected])})
        path = reverse(path_name)
        return HttpResponseRedirect(f"{path}?{params}")

    @admin.action(description=_("Export data via webhook"), permissions=['change'])
    def export_webhook(self, request: HttpRequest, queryset: QuerySet) -> HttpResponseRedirect:
        return self.process_response_redirect(queryset, "admin:webhook_export")

    @admin.action(description=_("Send data via webhook"), permissions=['change'])
    def send_webhook(self, request: HttpRequest, queryset: QuerySet) -> HttpResponseRedirect:
        return self.process_response_redirect(queryset, "admin:webhook_send")

    @admin.action(description=_("Set as spam"), permissions=['change'])
    def honeypot_filled_on(self, request: HttpRequest, queryset: QuerySet) -> HttpResponseRedirect:
        queryset.update(honeypot_filled=True)

    @admin.action(description=_("Set not to spam"), permissions=['change'])
    def honeypot_filled_off(self, request: HttpRequest, queryset: QuerySet) -> HttpResponseRedirect:
        queryset.update(honeypot_filled=False)



import logging
from typing import TYPE_CHECKING

from django.conf import settings
from django.contrib.sites.models import Site
from django.http import HttpRequest
from django.utils.translation import gettext_lazy as _

from .action_backends_base import BaseAction
from .api.webhook import trigger_webhooks
from .constants import ALDRYN_FORMS_MULTIPLE_SUBMISSION_DURATION


if TYPE_CHECKING:  # pragma: no cover
    from .cms_plugins import FormPlugin as CMSFormPlugin
    from .forms import FormSubmissionBaseForm
    from .models import FormPlugin


logger = logging.getLogger(__name__)


class DefaultAction(BaseAction):
    verbose_name = _('Save to site administration and send email')

    def form_valid(
        self,
        cmsplugin: "CMSFormPlugin",
        instance: "FormPlugin",
        request: HttpRequest,
        form: "FormSubmissionBaseForm"
    ) -> None:
        duration = getattr(settings, ALDRYN_FORMS_MULTIPLE_SUBMISSION_DURATION, 0)
        if duration:
            recipients = cmsplugin.postpone_send_notifications(instance, form)
            form.instance.set_recipients(recipients)
            submission = form.save()
            for hook in instance.webhooks.all():
                submission.webhooks.add(hook)
        elif not form.instance.honeypot_filled:
            recipients = cmsplugin.send_notifications(instance, form)
            form.instance.set_recipients(recipients)
            submission = form.save()
            for hook in instance.webhooks.all():
                submission.webhooks.add(hook)
            site = Site.objects.first()
            trigger_webhooks(instance.webhooks, form.instance, site.domain)
        cmsplugin.send_success_message(instance, request)


class EmailAction(BaseAction):
    verbose_name = _('Only send email')

    def form_valid(
        self,
        cmsplugin: "CMSFormPlugin",
        instance: "FormPlugin",
        request: HttpRequest,
        form: "FormSubmissionBaseForm"
    ) -> None:
        duration = getattr(settings, ALDRYN_FORMS_MULTIPLE_SUBMISSION_DURATION, 0)
        if duration:
            recipients = cmsplugin.postpone_send_notifications(instance, form)
        elif not form.instance.honeypot_filled:
            recipients = cmsplugin.send_notifications(instance, form)
            logger.info(f'Sent email notifications to {len(recipients)} recipients.')
            site = Site.objects.first()
            trigger_webhooks(instance.webhooks, form.instance, site.domain)
        cmsplugin.send_success_message(instance, request)


class NoAction(BaseAction):
    verbose_name = _('No action')

    def form_valid(
        self,
        cmsplugin: "CMSFormPlugin",
        instance: "FormPlugin",
        request: HttpRequest,
        form: "FormSubmissionBaseForm"
    ) -> None:
        form_id = form.form_plugin.id
        logger.info(f'Not persisting data for "{form_id}" since action_backend is set to "none"')



from django.db.models.signals import pre_delete
from django.dispatch import Signal, receiver

from .models import EmailFieldPlugin


form_pre_save = Signal()
form_post_save = Signal()


@receiver(pre_delete, sender=EmailFieldPlugin, dispatch_uid='aldryn_forms_pre_delete_field')
def protect_form_action_backend(sender, instance, using, **kwargs):
    plugin_form, action_backend = instance.get_parent_form_action_backend()
    if action_backend is not None:
        getattr(action_backend, "delete_field", lambda field, form: None)(instance, plugin_form)
